@@ -39,7 +39,6 @@ import orca.input_event as input_event
 import orca.messages as messages
 import orca.orca as orca
 import orca.orca_state as orca_state
-import orca.speech as speech
 import orca.settings as settings
 import orca.settings_manager as settings_manager
 import orca.structural_navigation as structural_navigation
@@ -304,14 +303,6 @@ class Script(default.Script):
         prefs.update(self.spellcheck.getPreferencesFromGUI())
         return prefs
 
-    def presentObject(self, obj, **args):
-        if not self._lastCommandWasStructNav:
-            super().presentObject(obj, **args)
-            return
-
-        utterances = self.speechGenerator.generateSpeech(obj, **args)
-        speech.speak(utterances)
-
     def panBrailleLeft(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
         entire document.
@@ -510,6 +501,7 @@ class Script(default.Script):
 
         if self.spellcheck.isSuggestionsItem(newLocusOfFocus) \
            and not self.spellcheck.isSuggestionsItem(oldLocusOfFocus):
+            orca.emitRegionChanged(newFocus)
             self.updateBraille(newLocusOfFocus)
             self.spellcheck.presentSuggestionListItem(includeLabel=True)
             return
@@ -530,9 +522,7 @@ class Script(default.Script):
                     for tab in child:
                         eventState = tab.getState()
                         if eventState.contains(pyatspi.STATE_SELECTED):
-                            utterances = \
-                                self.speechGenerator.generateSpeech(tab)
-                            speech.speak(utterances)
+                            self.presentObject(tab)
 
         # TODO - JD: This is a hack that needs to be done better. For now it
         # fixes the broken echo previous word on Return.
@@ -552,8 +542,10 @@ class Script(default.Script):
             isControlKey = mods & keybindings.CTRL_MODIFIER_MASK
             isShiftKey = mods & keybindings.SHIFT_MODIFIER_MASK
             if event_string in ["Up", "Down"] and isControlKey and not isShiftKey:
-                if self.utilities.displayedText(newLocusOfFocus):
-                    speech.speak(self.utilities.displayedText(newLocusOfFocus))
+                string = self.utilities.displayedText(newLocusOfFocus)
+                if string:
+                    voice = self.speechGenerator.voice(obj=newLocusOfFocus, string=string)
+                    self.speakMessage(string, voice=voice)
                     self.updateBraille(newLocusOfFocus)
                     try:
                         text = newLocusOfFocus.queryText()
@@ -640,10 +632,17 @@ class Script(default.Script):
                 self.spellcheck.presentErrorDetails()
             return
 
+        if self.utilities.isSpreadSheetCell(event.any_data) \
+           and not event.any_data.getState().contains(pyatspi.STATE_FOCUSED) \
+           and not event.source.getState().contains(pyatspi.STATE_FOCUSED) :
+            msg = "SOFFICE: Neither source nor child have focused state. Clearing cache on table."
+            debug.println(debug.LEVEL_INFO, msg, True)
+            event.source.clearCache()
+
         default.Script.onActiveDescendantChanged(self, event)
 
-    def onChildrenChanged(self, event):
-        """Callback for object:children-changed accessibility events."""
+    def onChildrenAdded(self, event):
+        """Callback for object:children-changed:add accessibility events."""
 
         if self.utilities.isSpreadSheetCell(event.any_data):
             orca.setLocusOfFocus(event, event.any_data)
@@ -669,7 +668,7 @@ class Script(default.Script):
             self.presentMessage(full, brief)
             return
 
-        default.Script.onChildrenChanged(self, event)
+        default.Script.onChildrenAdded(self, event)
 
     def onFocus(self, event):
         """Callback for focus: accessibility events."""
@@ -775,6 +774,19 @@ class Script(default.Script):
                 orca.setLocusOfFocus(event, event.source, False)
                 return
 
+        if self.utilities.isSpreadSheetTable(event.source) and orca_state.locusOfFocus:
+            if self.utilities.isDead(orca_state.locusOfFocus):
+                msg = "SOFFICE: Event believed to be post-editing focus claim. Dead locusOfFocus."
+                debug.println(debug.LEVEL_INFO, msg, True)
+                orca.setLocusOfFocus(event, event.source, False)
+                return
+
+            if  orca_state.locusOfFocus.getRole() in [pyatspi.ROLE_PARAGRAPH, pyatspi.ROLE_TABLE_CELL]:
+                msg = "SOFFICE: Event believed to be post-editing focus claim based on role."
+                debug.println(debug.LEVEL_INFO, msg, True)
+                orca.setLocusOfFocus(event, event.source, False)
+                return
+
         default.Script.onFocusedChanged(self, event)
 
     def onCaretMoved(self, event):
@@ -783,6 +795,13 @@ class Script(default.Script):
         if event.detail1 == -1:
             return
 
+        if event.source.getRole() == pyatspi.ROLE_PARAGRAPH \
+           and not event.source.getState().contains(pyatspi.STATE_FOCUSED):
+            event.source.clearCache()
+            if event.source.getState().contains(pyatspi.STATE_FOCUSED):
+                msg = "SOFFICE: Clearing cache was needed due to missing state-changed event."
+                debug.println(debug.LEVEL_INFO, msg, True)
+
         if self.utilities._flowsFromOrToSelection(event.source):
            return
 
@@ -832,7 +851,7 @@ class Script(default.Script):
             wasCommand = mods & keybindings.COMMAND_MODIFIER_MASK
             weToggledIt = wasCommand and keyString not in navKeys
         if weToggledIt:
-            speech.speak(self.speechGenerator.generateSpeech(obj))
+            self.presentObject(obj, alreadyFocused=True, interrupt=True)
 
     def onSelectedChanged(self, event):
         """Callback for object:state-changed:selected accessibility events."""
@@ -874,7 +893,9 @@ class Script(default.Script):
             return
 
         selectedChildren = self.utilities.selectedChildren(event.source)
-        if len(selectedChildren) == 1:
+        if len(selectedChildren) == 1 \
+           and self.utilities.containingComboBox(event.source) == \
+               self.utilities.containingComboBox(orca_state.locusOfFocus):
             orca.setLocusOfFocus(event, selectedChildren[0], True)
 
     def onTextSelectionChanged(self, event):
@@ -885,6 +906,11 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return
 
+        if self.utilities.isDead(event.source):
+            msg = "SOFFICE: Ignoring event from dead source."
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+
         if event.source != orca_state.locusOfFocus \
            and event.source.getState().contains(pyatspi.STATE_FOCUSED):
             orca.setLocusOfFocus(event, event.source, False)
